/*////////////////////////////////////////////////////////////////////////
Copyright (c) 1991 Electrotechnical Laboratry (ETL)

Permission to use, copy, modify, and distribute this material 
for any purpose and without fee is hereby granted, provided 
that the above copyright notice and this permission notice 
appear in all copies, and that the name of ETL not be 
used in advertising or publicity pertaining to this 
material without the specific, prior written permission 
of an authorized representative of ETL.
ETL MAKES NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY 
OF THIS MATERIAL FOR ANY PURPOSE.  IT IS PROVIDED "AS IS", 
WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES.
/////////////////////////////////////////////////////////////////////////
Content-Type:	program/C; charset=US-ASCII
Program:	xhist.c (XHIST command for NNTP)
Author:		Yutaka Sato <ysato@etl.go.jp>
Description:

    XHIST [<Message-ID>|selector]

    selector is list of:
	YYMMDD:HHMMSS[-YYMMDD:HHMMSS]	History range
	$				Last(latest) history line
	ENDLESS	[FILTER]		endless loop

#	/regular-expression/		History lines matches the exp.
#	+nnn				Ignore arts before nnn'th art.
#	-nnn				Ignore arts after nnn'th art.
#	! selector

    e.g.,
	XHIST <123@etl.go.jp>	retrieve subject of <123@etl.go.jp>
	XHIST 900701			retrieve since 1990 Jul. 1

	This command is an extention, and not included in RFC 977.

History:
	87xxxx	created as a part of vin (Bnews-2.11.17 format)
	900220	extracted to be a part of nntpd 
	900418	added Bnews-2.11.18 format
	900727	added "$" command
	910618	added Cnews arrivaldate-expiredate format
 V1.1	930522	added "ENDLESS" command
///////////////////////////////////////////////////////////////////////*/

#include "common.h"
#define INCLUDE_DONE

extern int (*xnntp_rstat)();
extern int (*xnntp_rbody)();
#define RSTAT	(*xnntp_rstat)
#define RBODY	(*xnntp_rbody)
extern char XHIST_LASTLINE_COMMAND[];

static char *usage =
"Usage: XHIST {<Message-Id>|YYMMDD:HHMMSS[-YYMMDD:HHMMSS]|$|ENDLESS}";
static putusg(){ RSTAT(ERR_CMDSYN,"%s",usage); }
#define ENDLESS	"ENDLESS"

xhist(ac,av)
	char *av[];
{	int i;

	if(ac <= 1){
		putusg();
		fflush(stdout);
	}else
	if( strcmp(av[1],"ENDLESS") == 0 ){
		xhist_endless(av[2]);
	}else
	switch(av[1][0]){
		default:
			if( strcmp(av[1],XHIST_LASTLINE_COMMAND) == 0 ){
				xhist_tail();
				break;
			}
			xhist_by_date_range(av[1]);
			break;
		case '<':
			xhist_by_message_id(av[1]);
			break;
		case '/':
			putusg();
			break;
	}
	fflush(stdout);
}

static xhist_tail(){
	char line[2048];

	historyLastLine(HISTORY_FILE,line);
	if( *line )
		RSTAT(OK_HEAD,  "%s",line);
	else	RSTAT(ERR_NOART,"Cant access last article");
}

static xhist_endless(filter)
	char *filter;
{	FILE *fp;
	char line[2048];

	fp = fopen(HISTORY_FILE,"r");
	if( fp == NULL )
		return putusg();

	comp_histfilter(filter);

	RSTAT(OK_HEAD,"selected history lines follow");
	fseek(fp,0,2);

	for(;;){
		if( fgets(line,sizeof(line),fp) == NULL ){
			clearerr(fp);
			/* check inode, reopen if changed */
			fseek(fp,0,2);
			sleep(5);
			continue;
		}
		if( exec_histfilter(line) == 0 )
			RBODY(line);
	}
}

#ifdef _IOFBF
#define SETBUFER(fp,buf,size)	setvbuf(fp,hbuf,_IOFBF,size)
#else
#define SETBUFER(fp,buf,size)	setbuffer(fp,hbuf,size)
#endif

static xhist_by_date_range(drange)
	char *drange;
{
	int fromdate,fromtime,todate,totime;
	char line[1000],hbuf[512];
	FILE *fp,*xfp;
	int offset;

	fromdate = 0; fromtime = 0;
	todate = 991231; totime = 246000;
	if(sscanf(drange,"%d:%d-%d:%d",&fromdate,&fromtime,&todate,&totime)==0){
		putusg();
		return;
	}

	fp = fopen(HISTORY_FILE,"r");
	if(fp == NULL){
		putusg();
		return;
	}

	RSTAT(OK_HEAD,"selected history lines follow");
	SETBUFER(fp,hbuf,sizeof(hbuf));

	offset = history_bsearchx(fp,fromdate,fromtime);
	xfp = fdopen(fileno(fp),"r");
	fseek(xfp,offset,0);
	while( fgets(line,sizeof(line),xfp) != NULL )
		RBODY(line);
	RBODY(0);
	fclose(xfp);
	fclose(fp);
}
static history_bsearchx(fp,xdate,xtime)
	FILE *fp;
{	int xclock;

	xclock = TimeGm(xdate/10000,(xdate/100)%100,xdate%100,
		      xtime/10000,(xtime/100)%100,xtime%100);
	return history_bsearch(fp,xclock);
}

static xhist_by_message_id(mid)
	char *mid;
{	char xmid[1000],line[10000];

	if(*mid != '<')
		sprintf(xmid,"<%s>",mid);
	else	strcpy(xmid,mid);

	if( 0 < historyByMessageID(HISTORY_FILE,xmid,line,sizeof(line)))
		RSTAT(OK_HEAD,  "%s",line);
	else	RSTAT(ERR_NOART,"No article by message-id %s",xmid);
}
